==> bless-0.3.1.tar.gz: bless-0.3.1/src/gui/areas/GroupedArea.cs <==
// created on 7/16/2004 at 7:54 PM
/*
 *   Copyright (c) 2004, Alexandros Frantzis (alf82 [at] freemail [dot] gr)
 *
 *   This file is part of Bless.
 *
 *   Bless is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Bless is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Bless; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
using System;
using Gtk;
using Gdk;
using Bless.Gui.Drawers;

namespace Bless.Gui.Areas {

///<summary>An area that displays grouped bytes</summary>
abstract public class GroupedArea : Area {
	
	int grouping;

	public GroupedArea(Gtk.DrawingArea da)
	: base(da)
	{
		grouping=1;
		canFocus=true;
	}
	
		
	public int Grouping {
		set {  grouping=value; }
		get {  return grouping; }
	
	}
	
	protected override void RenderRow(int i, int p, int n, bool blank, bool select) 
	{
		int rx=0+x;
		int ry=i*font.Height+y;
		long roffset=offset+i*bpr+p;
		bool odd;		
		
		// odd row?
		odd=(((roffset/bpr)%2)==1);
		
		if (blank==true) {
			if (odd)
				backPixmap.DrawRectangle(backOddGC, true, rx, ry, width, font.Height);
			else
				backPixmap.DrawRectangle(backEvenGC, true, rx, ry, width, font.Height);
		}
		
		Drawer.RowType rowType;
		Drawer.ColumnType colType;
		
		if (odd)
			rowType=Drawer.RowType.Odd;
		else
			rowType=Drawer.RowType.Even;		
				
		if (select)
		    colType=Drawer.ColumnType.Selected;
		else    
		    colType=Drawer.ColumnType.Even;
		    
		int pos=0;
		// draw bytes
		while (true) {
			
			if (pos>=p) { //don't draw until we reach p
				if (!select) {
					if ((pos/grouping)%2 == 0)
						colType=Drawer.ColumnType.Even;
					else
						colType=Drawer.ColumnType.Odd;
				}
				
				font.Draw(backEvenGC, backPixmap, rx, ry, byteBuffer[roffset++], rowType, colType);
				if (--n<=0)
					break;
			}
			
			// space if necessary
			if (pos%grouping == grouping-1) 
				rx=rx+(dpb+1)*font.Width;
			else 
				rx=rx+dpb*font.Width;
			
			pos++;
		}
	}

	public override int CalcWidth(int n) 
	{
		if (n==0)
			return 0;
		if (n % grouping != 0) // can't break the grouping
			return -1;
			
		int ngroups=n/grouping;
		int groupWidth=grouping*dpb*font.Width;
		
		return ngroups*groupWidth+(ngroups-1)*font.Width;
	}
	
	public override void GetDisplayInfoByOffset(long off, out int orow, out int obyte, out int ox, out int oy)
	{	 
		orow=(int)((off-offset)/bpr);
		obyte=(int)((off-offset)%bpr);
			
		oy=orow*font.Height;
	
		int group=obyte/grouping;
		int groupOffset=obyte%grouping;
		ox=group*(grouping*dpb*font.Width+font.Width) + dpb*font.Width*groupOffset;
	}
 
	public override long GetOffsetByDisplayInfo(int x, int y, out int digit, out GetOffsetFlags flags)
	{
		flags=0;
		int groupWidth=(grouping*dpb*font.Width+font.Width);

		int row=y/font.Height;
		int group=x/groupWidth;
		int groupByte=(x-group*groupWidth)/(dpb*font.Width);
		
		digit = (x - group*groupWidth - groupByte*dpb*font.Width)/font.Width;
		
		if (groupByte >= grouping) {
			groupByte=grouping-1;
			flags |= GetOffsetFlags.Abyss;
		}
		
		long off=row*bpr + (group*grouping+groupByte) + offset;
		if (off >= byteBuffer.Size)
			flags |= GetOffsetFlags.Eof;
			
		return off;
	
	}
	


}// end GroupedArea

}//end namespace

==> bless-0.3.4.tar.gz: bless-0.3.4/src/gui/areas/GroupedArea.cs <==
// created on 7/16/2004 at 7:54 PM
/*
 *   Copyright (c) 2004, Alexandros Frantzis (alf82 [at] freemail [dot] gr)
 *
 *   This file is part of Bless.
 *
 *   Bless is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Bless is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Bless; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
using System;
using Gtk;
using Gdk;
using Bless.Gui.Drawers;

namespace Bless.Gui.Areas {

///<summary>An area that displays grouped bytes</summary>
abstract public class GroupedArea : Area {
	
	int grouping;

	public GroupedArea()
	: base()
	{
		grouping=1;
		canFocus=true;
	}
	
		
	public int Grouping {
		set {  grouping=value; }
		get {  return grouping; }
	
	}
	
	protected override void RenderRow(int i, int p, int n, bool blank, bool select) 
	{
		int rx=0+x;
		int ry=i*drawer.Height+y;
		long roffset=offset+i*bpr+p;
		bool odd;		
		
		// odd row?
		odd=(((roffset/bpr)%2)==1);
		
		if (blank==true) {
			if (odd)
				backPixmap.DrawRectangle(backOddGC, true, rx, ry, width, drawer.Height);
			else
				backPixmap.DrawRectangle(backEvenGC, true, rx, ry, width, drawer.Height);
		}
		
		Drawer.RowType rowType;
		Drawer.ColumnType colType;
		
		if (odd)
			rowType=Drawer.RowType.Odd;
		else
			rowType=Drawer.RowType.Even;		
				
		if (select)
		    colType=Drawer.ColumnType.Selected;
		else    
		    colType=Drawer.ColumnType.Even;
		    
		int pos=0;
		// draw bytes
		while (true) {
			
			if (pos>=p) { //don't draw until we reach p
				if (!select) {
					if ((pos/grouping)%2 == 0)
						colType=Drawer.ColumnType.Even;
					else
						colType=Drawer.ColumnType.Odd;
				}
				
				drawer.Draw(backEvenGC, backPixmap, rx, ry, byteBuffer[roffset++], rowType, colType);
				if (--n<=0)
					break;
			}
			
			// space if necessary
			if (pos%grouping == grouping-1) 
				rx=rx+(dpb+1)*drawer.Width;
			else 
				rx=rx+dpb*drawer.Width;
			
			pos++;
		}
	}

	public override int CalcWidth(int n) 
	{
		if (n==0)
			return 0;
		if (fixedBpr > 0 && n>fixedBpr) // must adhere to fixed length
			return -1;
		if (n % grouping != 0) // can't break the grouping
			return -1;
		
		int ngroups=n/grouping;
		int groupWidth=grouping*dpb*drawer.Width;
		
		return ngroups*groupWidth+(ngroups-1)*drawer.Width;
	}
	
	public override void GetDisplayInfoByOffset(long off, out int orow, out int obyte, out int ox, out int oy)
	{	 
		orow=(int)((off-offset)/bpr);
		obyte=(int)((off-offset)%bpr);
			
		oy=orow*drawer.Height;
	
		int group=obyte/grouping;
		int groupOffset=obyte%grouping;
		ox=group*(grouping*dpb*drawer.Width+drawer.Width) + dpb*drawer.Width*groupOffset;
	}
 
	public override long GetOffsetByDisplayInfo(int x, int y, out int digit, out GetOffsetFlags flags)
	{
		flags=0;
		int groupWidth=(grouping*dpb*drawer.Width+drawer.Width);

		int row=y/drawer.Height;
		int group=x/groupWidth;
		int groupByte=(x-group*groupWidth)/(dpb*drawer.Width);
		
		digit = (x - group*groupWidth - groupByte*dpb*drawer.Width)/drawer.Width;
		
		if (groupByte >= grouping) {
			groupByte=grouping-1;
			flags |= GetOffsetFlags.Abyss;
		}
		
		long off=row*bpr + (group*grouping+groupByte) + offset;
		if (off >= byteBuffer.Size)
			flags |= GetOffsetFlags.Eof;
			
		return off;
	
	}
	


}// end GroupedArea

}//end namespace
